---
layout: compress
permalink: /assets/js/site.js
---

{% include scripts/jquery/jquery-3.3.1.custom.min.js %}
{% include scripts/jquery-address/jquery-address-1.6.semantic-ui.min.js %}

{% include scripts/mousetrap/mousetrap-1.6.1.min.js %}

{% include scripts/semantic-ui/_version.js %}
{% include scripts/semantic-ui/site.min.js %}

{% include scripts/semantic-ui/dimmer.min.js %}
{% include scripts/semantic-ui/modal.min.js %}
{% include scripts/semantic-ui/search.min.js %}
{% include scripts/semantic-ui/sidebar.min.js %}
{% include scripts/semantic-ui/sticky.min.js %}
{% include scripts/semantic-ui/tab.min.js %}
{% include scripts/semantic-ui/transition.min.js %}
