query ($ownName: String!, $repoName: String!, $numUsers: Int!, $pgCursor: String) {
  repository(owner: $ownName, name: $repoName) {
    stargazers(first: $numUsers, after: $pgCursor) {
      edges {
        starredAt
      }
      pageInfo {
        endCursor
        hasNextPage
      }
    }
    name
    nameWithOwner
  }
}
# {"ownName": "LLNL", "repoName": "llnl.github.io", "numUsers": 100, "pgCursor": null}
